﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace CommandDesignPattern
{
    class User
    {
        // Initializers
        private Calculator _calculator = new Calculator();
        private List<ICommand> _commands = new List<ICommand>();
        private int _currentIndex = 0;

        public void RedoCalculation(int levels)
        {
            Console.WriteLine("\n---- Redo {0} levels ", levels);
            // Perform redo operations
            for (int i = 0; i < levels; i++)
            {
                if (_currentIndex < _commands.Count - 1)
                {
                    ICommand command = _commands[_currentIndex++];
                    command.Execute();
                }
            }
        }

        public void UndoCalculation(int levels)
        {
            Console.WriteLine("\n---- Undo {0} levels ", levels);
            // Perform undo operations
            for (int i = 0; i < levels; i++)
            {
                if (_currentIndex > 0)
                {
                    ICommand command = _commands[--_currentIndex] as ICommand;
                    command.UnExecute();
                }
            }
        }

        public void Calculate(char @operator, int operand)
        {
            // Create command operation and execute it
            ICommand command = new CalculationCommand(
              _calculator, @operator, operand);
            command.Execute();

            // Add command to undo list
            _commands.Add(command);
            _currentIndex++;
        }
    }
}
